#!/usr/bin/perl -w

# Copyright 2012 Kevin Ryde

# This file is part of Math-Image.
#
# Math-Image is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# Math-Image is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with Math-Image.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use warnings;

use Smart::Comments;


{
  require Compress::Zlib;
  my $fh = Compress::Zlib::gzopen("lib/App/MathImage/pi.gz", "r")
    || die;
  $fh->gzseek (0,
               0)  # SEEK_SET
    or die "cannot seek";

  $fh->gzseek (2,
               0)  # SEEK_SET
    or die "cannot seek";

  $fh->gzseek (0,
               0)  # SEEK_SET
    or die "cannot seek";

  exit 0;
}
